import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime as dt
import sys
from KEMP import Basic_FDTD, Dielectric, to_epr, to_SI, to_NU, Gold, Silver, PA
from KEMP import structures as stc

nm = 1e-9
dx, dy, dz = [5*nm, 5*nm, 5*nm]
nx, ny, nz = [64, 64, 600]
x = np.ones(nx, dtype=np.float64)*dx # 0 ~ 3000 nm
y = np.ones(ny, dtype=np.float64)*dy # 0 ~ 100 nm
z = np.ones(nz, dtype=np.float64)*dz # 0 ~ 100 nm
lx, ly, lz = dx*nx, dy*ny, dz*nz
space_grid = (x, y ,z)
#engine_name = 'nvidia_cuda'
engine_name = 'intel_cpu'
fdtd = Basic_FDTD('3D', space_grid, dtype=np.complex64, engine=engine_name, device_id=2)

pml_apply = {'x':'' , 'y':''  , 'z':'+-'  }
pbc_apply = {'x':True, 'y':True, 'z':False}
fdtd.apply_PML(pml_apply)
fdtd.apply_PBC(pbc_apply)


free = Dielectric(to_epr(fdtd, n=1))
HSQ  = Dielectric(to_epr(fdtd, n=1.5))

slab1  = stc.Box(Silver, ((0.*nm,0.*nm,700*nm),(320*nm,320*nm,730*nm)))
hole1  = stc.Box(free,((60.*nm,60.*nm,700*nm),(260*nm,260*nm,730*nm)))

slab2  = stc.Box(HSQ, ((0.*nm,0.*nm,730*nm),(320*nm,320*nm,765*nm)))
hole2  = stc.Box(free,((60.*nm,60.*nm,730*nm),(260*nm,260*nm,765*nm)))

slab3  = stc.Box(Silver, ((0.*nm,0.*nm,765*nm),(320*nm,320*nm,795*nm)))
hole3  = stc.Box(free,((60.*nm,60.*nm,765*nm),(260*nm,260*nm,795*nm)))

structures = [slab1,hole1,slab2,hole2,slab3,hole3]
#structures = [slab2]

from scipy.constants import c
wavelength = np.arange(400, 1001, 5)*nm
freqs = c/wavelength
freq0 = c/(500.*nm)
wfreq0_NU = to_NU(fdtd, 'angular frequency', 2.*np.pi*freq0)

src_pos= 30     # source position is 30*dz along z-axis
trs_pos= -30    # measuring position for transmittance is -30*dz along z-axis

inc    = fdtd.apply_direct_source('ey', ((0,0,src_pos),(-1,-1,src_pos)))
fdtd.set_structures(structures)

print('Setting Complete and READY to RUN')

plt.ion()
t0 = dt.now()
tc = 500
ts = 50.
tmax = 30000 # 5 period for 1000 nm wavelength


#ex_trs = np.zeros(tmax, dtype=np.complex64)

ey_ref = np.zeros(tmax, dtype=np.complex64)
ey_trs = np.zeros(tmax, dtype=np.complex64)

#ez_trs = np.zeros(tmax, dtype=np.complex64)

hx_ref = np.zeros(tmax, dtype=np.complex64)
hx_trs = np.zeros(tmax, dtype=np.complex64)

#hy_trs = np.zeros(tmax, dtype=np.complex64)
#hz_trs = np.zeros(tmax, dtype=np.complex64)

#sx_ref = np.zeros(tmax, dtype=np.complex64)
#sx_trs = np.zeros(tmax, dtype=np.complex64)

#sy_ref = np.zeros(tmax, dtype=np.complex64)
#sy_trs = np.zeros(tmax, dtype=np.complex64)

sz_trs = np.zeros(tmax, dtype=np.complex64)
sz_ref = np.zeros(tmax, dtype=np.complex64)

for tstep in xrange(tmax):
    src = np.exp(-(tstep-tc)**2/(ts**2))*np.exp(1.j*wfreq0_NU*(tstep-tc)*fdtd.dt)

    ey_ref[tstep] = fdtd.ey[:,:,src_pos].mean() - src
    ey_trs[tstep] = fdtd.ey[:,:,trs_pos].mean()
    hx_ref[tstep] = fdtd.hx[:,:,src_pos].mean()
    hx_trs[tstep] = fdtd.hx[:,:,trs_pos].mean()

    inc.set_source(src)
    fdtd.updateE()
    fdtd.updateH()


    if tstep % 1000 == 0:
        t1 = dt.now()

        print('%s %05d time step\r' % (t1-t0, tstep)),

        sys.stdout.flush()

        '''
        plt.clf()
        plt.plot(fdtd.ey.real[nx/2,ny/2,:], 'r')
        plt.plot(fdtd.ey.imag[nx/2,ny/2,:], 'b')
        plt.ylim(-2., 2.)
        plt.draw()
        '''
t1 = dt.now()
print (t1-t0, 'Simulation Complete')

dt_SI  = to_SI(fdtd, 'time', fdtd.dt)
tsteps = np.arange(tmax
    , dtype=fdtd.dtype)
t      = tsteps*dt_SI

ey_src = np.exp(-(tsteps-tc)**2/(ts**2))*np.exp(1.j*wfreq0_NU*(tsteps-tc)*fdtd.dt)
hx_src = np.exp(-(tsteps-tc)**2/(ts**2))*np.exp(1.j*wfreq0_NU*(tsteps-tc)*fdtd.dt)/c

ey_src_ft = ((dt_SI/(2.*np.pi))*ey_src[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
ey_ref_ft = ((dt_SI/(2.*np.pi))*ey_ref[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
ey_trs_ft = ((dt_SI/(2.*np.pi))*ey_trs[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)

hx_ref_ft = ((dt_SI/(2.*np.pi))*hx_ref[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
hx_trs_ft = ((dt_SI/(2.*np.pi))*hx_trs[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
hx_src_ft = ((dt_SI/(2.*np.pi))*hx_src[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
sz_trs_ft = -ey_trs_ft*np.conjugate(hx_trs_ft)


Trans = abs(ey_trs_ft)**2/abs(ey_src_ft)**2
Reflec= abs(ey_ref_ft)**2/abs(ey_src_ft)**2
Poyntz= abs(sz_trs_ft)

print("T = ", Trans)
print("R = ", Reflec)
print("T + R = " ,(Trans + Reflec))
print("Sz = " ,Poyntz)

print("lambda = %3e" , wavelength)


plt.plot(wavelength*1e9,Trans,label='Transmittance')
plt.plot(wavelength*1e9,Reflec,label='Reflectance')
plt.legend(loc = 'center right')
plt.xlabel("nm")
plt.ylabel("Ratio")
plt.show()
plt.savefig("Fishnet for Chiral")

import time as t
#t.sleep(5)

"""
import h5py as h5
f = h5.File('./save/%s.h5' %'Chiral','w')
f.create_dataset('ey_trs', data=ey_trs_ft)
f.create_dataset('ey_ref', data=ey_ref_ft)
f.close()
"""
